/*
 * This example requires .NET Framework 4.5 or above.
 *
 */
using System;
using System.Net.Http;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace ClientCredentialsGrant
{
	class Program
	{
		public static void Main(string[] args)
		{
			if (args.Length != 2) {
				Console.WriteLine("Usage: ClientCredentialsGrant.exe apiKey apiSecret");
				Environment.Exit(1);
			}
			string clientId = args[0];
			string clientSecret = args[1];
			
			var postData = new List<KeyValuePair<string, string>>();
            postData.Add(new KeyValuePair<string, string>("client_id", clientId));
            postData.Add(new KeyValuePair<string, string>("client_secret", clientSecret));
            postData.Add(new KeyValuePair<string, string>("grant_type", "client_credentials"));
                                      
            HttpContent content = new FormUrlEncodedContent(postData);                         
			
            getToken(content);
            Console.ReadKey(true);
		}
		
		private static async void getToken(HttpContent content)
		{
			var result = await PostRequestAsync(content);
			
			var ser = new JavaScriptSerializer();
			var jsonResult = ser.Deserialize<Dictionary<string, string>>(result);
			string accessToken = jsonResult["access_token"];
			string expiresIn= jsonResult["expires_in"];
			Console.WriteLine("access token: " + accessToken);
			Console.WriteLine("expires in: " + expiresIn); //in seconds
			
			int expiresAt = getExpirationTime(expiresIn);
			
			//
			var currentTimeStamp = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds;
			if (currentTimeStamp >= expiresAt) {
				result = await PostRequestAsync(content);
				jsonResult = ser.Deserialize<Dictionary<string, string>>(result);
				accessToken = jsonResult["access_token"];
				expiresIn= jsonResult["expires_in"];
				Console.WriteLine("access token: " + accessToken);
			    Console.WriteLine("expires in: " + expiresIn);
			}
		}
		
		private static async Task<string> PostRequestAsync(HttpContent content) 
		{
			const string stsUrl = "https://login.myacxiom.com/api/v1/auth/oauth2/token";
			
			var client = new HttpClient();
			HttpResponseMessage response = await client.PostAsync(stsUrl, content);
			response.EnsureSuccessStatusCode();

            string result = await response.Content.ReadAsStringAsync();
            Console.WriteLine(result);
            
            return result;
		}
		
		private static int getExpirationTime(string expiresIn)
		{
			const int bufferTime = 60; //leave some buffer time before the current token expires
			var currentTimeStamp = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds;
			var expiresAt = currentTimeStamp + Int32.Parse(expiresIn) - bufferTime;
			Console.WriteLine("token is expiring at: " + expiresAt); //timestamp
			
			return expiresAt;
		}
	}
	
	
}