/*
 *  This example requires .NET Framework 4.5 or above.
 * 
 */
using System;
using System.Net.Http;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace PasswordGrant
{
	class Program
	{
		public static void Main(string[] args)
		{
			if (args.Length != 5) {
				Console.WriteLine("Usage: PasswordGrant.exe client_id, client_secret, tenant_system_name, user_name, password");
				Environment.Exit(1);
			}
			string clientId = args[0];
			string clientSecret = args[1];
			string tenantSystemName = args[2];
			string userName = args[3];
			string password = args[4];
			
			var data = new Dictionary<string, string>();
            data.Add("client_id", clientId);
            data.Add("client_secret", clientSecret);
            data.Add("userName", tenantSystemName+"\\"+userName);
            data.Add("password", password);
            data.Add("grant_type", "password");                     
			
            getToken(data);
            Console.ReadKey(true);
		}
		
		private static async void getToken(Dictionary<string, string> data)
		{
			HttpContent content = new FormUrlEncodedContent(data);
			var result = await PostRequestAsync(content);
			
			var ser = new JavaScriptSerializer();
			var jsonResult = ser.Deserialize<Dictionary<string, string>>(result);
			string accessToken = jsonResult["access_token"];
			string expiresIn= jsonResult["expires_in"];
			string refreshToken = jsonResult["refresh_token"];
			Console.WriteLine("access token: " + accessToken);
			Console.WriteLine("expires in: " + expiresIn); //in seconds
			Console.WriteLine("refresh token: " + refreshToken);
			
			int expiresAt = getExpirationTime(expiresIn);
			
			var currentTimeStamp = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds;
			if (currentTimeStamp >= expiresAt) {
				result = await renewToken(data["client_id"], data["client_secret"], refreshToken);
				jsonResult = ser.Deserialize<Dictionary<string, string>>(result);
				accessToken = jsonResult["access_token"];
				expiresIn= jsonResult["expires_in"];
				refreshToken = jsonResult["refresh_token"];
				Console.WriteLine("access token: " + accessToken);
			    Console.WriteLine("expires in: " + expiresIn);
			    Console.WriteLine("refresh token: " + refreshToken);
			}
		}
		
		private static async Task<string> renewToken(string clientId, string clientSecret, string refreshToken)
		{
			var refreshRequest = new Dictionary<string, string>();
			refreshRequest.Add("client_id", clientId);
			refreshRequest.Add("client_secret", clientSecret);
			refreshRequest.Add("grant_type", "refresh_token");
			refreshRequest.Add("refresh_token", refreshToken);
			
            var result = await PostRequestAsync(new FormUrlEncodedContent(refreshRequest));
            
            return result;
		}
		
		private static async Task<string> PostRequestAsync(HttpContent content) 
		{
			const string stsUrl = "https://login.myacxiom.com/api/v1/auth/oauth2/token";
			
			var client = new HttpClient();
			HttpResponseMessage response = await client.PostAsync(stsUrl, content);
			response.EnsureSuccessStatusCode();

            string result = await response.Content.ReadAsStringAsync();
            Console.WriteLine(result);
            
            return result;
		}
		
		private static int getExpirationTime(string expiresIn)
		{
			const int bufferTime = 60;
			var currentTimeStamp = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds;
			var expiresAt = currentTimeStamp + Int32.Parse(expiresIn) - bufferTime;
			Console.WriteLine("Token expiration time: " + expiresAt);
			
			return expiresAt;
		}
	}
}
